/* Copyright (c) 2000 John Stracke <fanorona@thibault.org>
 #
 * This file is part of Fanorona.
 #
 * Fanorona is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any later
 * version. 
 #
 * Fanorona is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
 * the GNU General Public License for more details. 
 #
 * You should have received a copy of the GNU General Public License along
 * with program; see the file COPYING. If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <sys/param.h>

#include <gnome.h>
#include <libgnome/gnome-config.h>

#include "prefs.h"

#define CONFIG_FILE "/fanorona/"
static const char file[]=CONFIG_FILE;
static const char prefix[]=CONFIG_FILE "fanorona/";

const char* getPrefWithDef(const char* name,
                           const char* defValue)
{
  const char* res;
  gboolean def;

  gnome_config_push_prefix(prefix);
  res=gnome_config_get_string_with_default(name,&def);
  gnome_config_pop_prefix();

  return def ? defValue : res;
}

const char* getPref(const char* name)
{
  const char* res;
  gnome_config_push_prefix(prefix);
  res=gnome_config_get_string(name);
  gnome_config_pop_prefix();
  return res;
}

long getPrefInt(const char* name,
                int defValue)
{
  long res;
  gboolean def;

  gnome_config_push_prefix(prefix);
  res=gnome_config_get_int_with_default(name,&def);
  gnome_config_pop_prefix();

  return def ? defValue : res;
}

int setPref(const char* name,
            const char* value)
{
  gnome_config_push_prefix(prefix);
  gnome_config_set_string(name,value);
  gnome_config_pop_prefix();
  return 0;
}

int setPrefInt(const char* name,
               long value)
{
  gnome_config_push_prefix(prefix);
  gnome_config_set_int(name,value);
  gnome_config_pop_prefix();
  return 0;
}

void clearPref(const char* name)
{
  setPref(name,"");
}

int savePrefs(void)
{
  /* For some reason, gnome_config_sync_file() takes char* rather than
   *  const char*.
   */
  char buff[sizeof(prefix)];
  strcpy(buff,file);
  gnome_config_sync_file(buff);
  return 0;
}

int loadPrefs(void)
{
  return 0;
}
