/* Copyright (c) 2000 John Stracke <fanorona@thibault.org>
 #
 * This file is part of Fanorona.
 #
 * Fanorona is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any later
 * version. 
 #
 * Fanorona is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
 * the GNU General Public License for more details. 
 #
 * You should have received a copy of the GNU General Public License along
 * with program; see the file COPYING. If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

#ifndef _FANORONA_BOARD_H_
#define _FANORONA_BOARD_H_

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdio.h>

#define FANORONA_NUM_ROWS 5
#define FANORONA_NUM_COLS 9

typedef enum {
  fanoronaBoardCellStateInvalid=0,
  fanoronaBoardCellStateEmpty,
  fanoronaBoardCellStatePlayer1,
  fanoronaBoardCellStatePlayer2
} FanoronaBoardCellState;

typedef struct _FanoronaBoard FanoronaBoard;
typedef struct _FanoronaBoardWatcher* FanoronaBoardWatcher;

struct _FanoronaBoard {
  FanoronaBoardCellState cells[FANORONA_NUM_ROWS][FANORONA_NUM_COLS];
  int numForPlayer1,numForPlayer2;
  FanoronaBoardWatcher watchers;
  int blockWatchers;
};

typedef void (*FanoronaBoardCellCallback)(FanoronaBoard* board,
                                          int col,
                                          int row,
                                          FanoronaBoardCellState state,
                                          void* context);
struct _FanoronaBoardWatcher {
  FanoronaBoardCellCallback callback;
  void* context;  
  FanoronaBoardWatcher next;
  FanoronaBoardWatcher prev;
};

FanoronaBoardCellState
fanoronaBoardCellOppositeState(FanoronaBoardCellState state);

int fanoronaBoardInit(FanoronaBoard* board);
int fanoronaBoardSetup(FanoronaBoard* board);

int fanoronaBoardCountSides(const FanoronaBoard* board,
                            int* numForPlayer1,
                            int* numForPlayer2);
int fanoronaBoardGameOver(const FanoronaBoard* board);

FanoronaBoardWatcher
fanoronaBoardAddWatcher(FanoronaBoard* board,
                        FanoronaBoardCellCallback callback,
                        void* context);
void
fanoronaBoardDelWatcher(FanoronaBoard* board,
                        FanoronaBoardWatcher watcher);

int fanoronaBoardSetCell(FanoronaBoard* board,
                         int col,
                         int row,
                         FanoronaBoardCellState state);
FanoronaBoardCellState fanoronaBoardGetCell(const FanoronaBoard* board,
                                            int col,
                                            int row);
int fanoronaBoardTestAdjacent(int col1, int row1,
                              int col2, int row2);
int fanoronaBoardIsLegalPos(int col, int row);

FanoronaBoard* fanoronaBoardNew(void);
int fanoronaBoardCopy(FanoronaBoard* dest,
                      const FanoronaBoard* src);
FanoronaBoard* fanoronaBoardClone(const FanoronaBoard* board);
void fanoronaBoardFree(FanoronaBoard* board);

FanoronaBoard* fanoronaBoardCloneAndSet(FanoronaBoard* board,
                                        int col,
                                        int row,
                                        FanoronaBoardCellState state);

int fanoronaBoardMove(FanoronaBoard* board,
                      int colFrom,
                      int rowFrom,
                      int colTo,
                      int rowTo,
                      int byApproach,
                      int* numCaptured);
int fanoronaBoardCopyAndMove(FanoronaBoard* dest,
                             const FanoronaBoard* src,
                             int colFrom,
                             int rowFrom,
                             int colTo,
                             int rowTo,
                             int byApproach,
                             int* numCaptured);

void fanoronaBoardPrint(FILE* out,
                        const FanoronaBoard* board);

#ifdef  __cplusplus
}
#endif

#endif /*_FANORONA_BOARD_H_*/
