/* Copyright (c) 2000 John Stracke <fanorona@thibault.org>
 *
 * This file is part of Fanorona.
 *
 * Fanorona is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any later
 * version. 
 *
 * Fanorona is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
 * the GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with program; see the file COPYING. If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

#include <iostream.h>

#include <fanorona/fltk.H>

#include <FL/fl_draw.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Return_Button.H>

FanoronaFlBoard::Cell::Cell(int x, int y,
                            int w, int h,
                            FanoronaFlBoard& _parent,
                            int _col, int _row)
  : Fl_Widget(x,y,w,h),
    parent(_parent),
    col(_col), row(_row),
    curMark(Unmarked)
{
}

void FanoronaFlBoard::Cell::mark(CellMark _mark)
{
  curMark=_mark;
  redraw();
}

FanoronaBoard& FanoronaFlBoard::getBoard()
{
  FanoronaBoard* board=fanoronaPlayerGetBoard(player);
  return *board;
}

void FanoronaFlBoard::Cell::draw()
{
  int color;
  FanoronaBoard& board=parent.getBoard();
  FanoronaBoardCellState state;

  state=fanoronaBoardGetCell(&board,col,row);

  switch (state)
    {
    case fanoronaBoardCellStateEmpty:
      color=FL_WHITE;
      break;

    case fanoronaBoardCellStatePlayer1:
      color=FL_RED;
      break;

    case fanoronaBoardCellStatePlayer2:
      color=FL_GREEN;
      break;
    }

  fl_clip(x(),y(),w(),h());

  {
    int x1=x();
    int x2=x1+w();
    int y1=y();
    int y2=y1+h();
    
    fl_color(color);
    fl_polygon(x1,y1,x1,y2,x2,y2,x2,y1);
  }

  fl_color((curMark==Possible) ? FL_WHITE : FL_BLACK);
  fl_rect(x(),y(),w(),h());

  fl_pop_clip();
}

void FanoronaFlBoard::draw()
{
  FanoronaBoardCellState state;

  fl_clip(x(),y(),w(),h());

  fl_color(FL_BLACK);
  fl_line(cellCenters.topleft.x,cellCenters.topleft.y,
          cellCenters.botmid.x,cellCenters.botmid.y);
  fl_line(cellCenters.botmid.x,cellCenters.botmid.y,
          cellCenters.topright.x,cellCenters.topright.y);
  fl_line(cellCenters.botleft.x,cellCenters.botleft.y,
          cellCenters.topmid.x,cellCenters.topmid.y);
  fl_line(cellCenters.topmid.x,cellCenters.topmid.y,
          cellCenters.botright.x,cellCenters.botright.y);

  fl_line((cellCenters.topleft.x+cellCenters.topmid.x)/2,
          cellCenters.topleft.y,
          cellCenters.midleft.x,cellCenters.midleft.y);
  fl_line((cellCenters.botleft.x+cellCenters.botmid.x)/2,
          cellCenters.botleft.y,
          cellCenters.midleft.x,cellCenters.midleft.y);
  fl_line((cellCenters.topright.x+cellCenters.topmid.x)/2,
          cellCenters.topright.y,
          cellCenters.midright.x,cellCenters.midright.y);
  fl_line((cellCenters.botright.x+cellCenters.botmid.x)/2,
          cellCenters.botright.y,
          cellCenters.midright.x,cellCenters.midright.y);

  fl_line((cellCenters.topleft.x+cellCenters.topmid.x)/2,
          cellCenters.topleft.y,
          (cellCenters.botmid.x+cellCenters.botright.x)/2,
          cellCenters.botright.y);
  fl_line((cellCenters.topright.x+cellCenters.topmid.x)/2,
          cellCenters.topright.y,
          (cellCenters.botmid.x+cellCenters.botleft.x)/2,
          cellCenters.botleft.y);

  for (int col=0; col<FANORONA_NUM_COLS; col++)
    {
      int x=cellCenters.topleft.x+col*(cellSteps.x);
      fl_line(x,cellCenters.topleft.y,
              x,cellCenters.botleft.y);
    }

  for (int row=0; row<FANORONA_NUM_ROWS; row++)
    {
      int y=cellCenters.topleft.y+row*(cellSteps.y);
      fl_line(cellCenters.topleft.x,y,
              cellCenters.topright.x,y);
    }

  fl_pop_clip();

  Fl_Group::draw();
}

extern "C" {
  void boardCellChanged(FanoronaBoard* board,
                        int col,
                        int row,
                        FanoronaBoardCellState state,
                        void* context);
}

FanoronaFlBoard::FanoronaFlBoard(int x, int y,
                                 int w, int h,
                                 const char* label)
  : Fl_Group(x,y,w,h,label),
    boardWatcher(0),
    disabled(0),
    delay(500)
{
  int pad=5;
  int cellWidth=(w+pad)/FANORONA_NUM_COLS-pad;
  int cellHeight=(h+pad)/FANORONA_NUM_ROWS-pad;
  int xoffset=x;
  int yoffset=y;

  player=fanoronaPlayerNew(0,2);

  if (cellHeight<cellWidth) {
    xoffset+=((cellWidth-cellHeight)*FANORONA_NUM_COLS)/2;
    cellWidth=cellHeight;
  } else {
    yoffset+=((cellHeight-cellWidth)*FANORONA_NUM_ROWS)/2;
    cellHeight=cellWidth;
  }

  for (int col=0; col<FANORONA_NUM_COLS; col++)
    {
      for (int row=0; row<FANORONA_NUM_ROWS; row++)
        {
          cells[col][row]
            =new Cell(xoffset+col*(cellWidth+pad),yoffset+row*(cellHeight+pad),
                      cellWidth,cellHeight,
                      *this,
                      col,row);
        }
    }

  int half=cellHeight/2;

  cellCenters.topleft.x=xoffset+half;
  cellCenters.topleft.y=yoffset+half;

  cellCenters.topmid.x=cellCenters.topleft.x+(cellWidth+pad)*4;
  cellCenters.topmid.y=cellCenters.topleft.y;

  cellCenters.topright.x=cellCenters.topleft.x+(cellWidth+pad)*8;
  cellCenters.topright.y=cellCenters.topleft.y;

  cellCenters.midleft.x=cellCenters.topleft.x;
  cellCenters.midleft.y=cellCenters.topleft.y+(cellHeight+pad)*2;

  cellCenters.midmid.x=cellCenters.topmid.x;
  cellCenters.midmid.y=cellCenters.midleft.y;

  cellCenters.midright.x=cellCenters.topright.x;
  cellCenters.midright.y=cellCenters.midleft.y;

  cellCenters.botleft.x=cellCenters.topleft.x;
  cellCenters.botleft.y=cellCenters.topleft.y+(cellHeight+pad)*4;

  cellCenters.botmid.x=cellCenters.topmid.x;
  cellCenters.botmid.y=cellCenters.botleft.y;

  cellCenters.botright.x=cellCenters.topright.x;
  cellCenters.botright.y=cellCenters.botleft.y;

  cellSteps.x=cellWidth+pad;
  cellSteps.y=cellHeight+pad;

  moveState.reset();
 
  end();

  boardWatcher=fanoronaBoardAddWatcher(&(getBoard()),
                                       boardCellChanged,
                                       this);
}

void FanoronaFlBoard::markChoice(int col, int row,
                                 Direction direction)
{
  int marked=0;
  switch (direction)
    {
    case Approach:
      moveState.pendingApproach.col=col;
      moveState.pendingApproach.row=row;
      marked=1;
      break;

    case Withdrawal:
      moveState.pendingWithdrawal.col=col;
      moveState.pendingWithdrawal.row=row;
      marked=1;
      break;
    }

  Cell* cell=cells[col][row];
  if (cell)
    cell->mark(marked ? Possible : Unmarked);
}

void FanoronaFlBoard::eraseChoices()
{
  markChoice(moveState.pendingApproach.col,
             moveState.pendingApproach.row,
             Invalid);
  markChoice(moveState.pendingWithdrawal.col,
             moveState.pendingWithdrawal.row,
             Invalid);
}

static void moveComputer(void* context)
{
  FanoronaFlBoard* board=(FanoronaFlBoard*)context;

  //cout << "moveComputer()\n";

  board->moveComputer();
}

void FanoronaFlBoard::moveComputer()
{
  int colFrom,rowFrom,colTo,rowTo,byApproach,turnIsOver;

  fanoronaPlayerMakeMove(player,
                         &colFrom,
                         &rowFrom,
                         &colTo,
                         &rowTo,
                         &byApproach,
                         &turnIsOver);

  if (turnIsOver)
    {
      disabled=0;

      {
        int numForPlayer1,numForPlayer2;

        fanoronaBoardCountSides(&(getBoard()),
                                &numForPlayer1,
                                &numForPlayer2);
        if ((!numForPlayer1) || (!numForPlayer2))
          gameOver(numForPlayer1>0);
        else
          whoseTurn(1);
      }
    }
  else
    {
      //cout << "add_timeout(" << delay << ")\n";
      Fl::add_timeout(delay,::moveComputer,(void*)this);
    }
}

void FanoronaFlBoard::cellClicked(Cell& cell,
                                  int col,
                                  int row)
{
  int turnIsOver=-1;

  //cout << "cellClicked(" << col << "," << row << ")\n";

  if (disabled)
    {
      //cout << "\tdisabled\n";
      return;
    }

  if (moveState.pendingApproach.col>=0)
    {
      //cout << "\tpendingApproach\n";

      Direction dir;

      if (   (moveState.pendingApproach.col==col)
             && (moveState.pendingApproach.row==row)
             )
        {
          //cout << "\tchose approach\n";
          dir=Approach;
        }
      else
        {
          if (   (moveState.pendingWithdrawal.col==col)
                 && (moveState.pendingWithdrawal.row==row)
                 )
            {
              //cout << "\tchose withdrawal\n";
              dir=Withdrawal;
            }
          else
            {
              //cout << "\tinvalid choice\n";
              return;
            }
        }

      fanoronaPlayerMoveMade(player,
                             moveState.pendingFrom.col,
                             moveState.pendingFrom.row,
                             moveState.pendingTo.col,
                             moveState.pendingTo.row,
                             (dir==Approach),
                             &turnIsOver);

      eraseChoices();

      int colTo=moveState.pendingTo.col;
      int rowTo=moveState.pendingTo.row;

      moveState.reset();

      if (!turnIsOver)
        {
          moveState.pendingFrom.col=colTo;
          moveState.pendingFrom.row=rowTo;
          cells[colTo][rowTo]->mark(Possible);
        }
    }
  else
    {
      if (moveState.pendingFrom.col>=0)
        {
          //cout << "\tpendingFrom\n";

          int canApproach,canWithdraw;
          if (!fanoronaPlayerOtherPlayerMoveValid(player,
                                                  moveState.pendingFrom.col,
                                                  moveState.pendingFrom.row,
                                                  col,row,
                                                  &canApproach,
                                                  &canWithdraw)
              )
            {
              //cout << "\tinvalid move\n";
              return;
            }

          cells[moveState.pendingFrom.col][moveState.pendingFrom.row]
            ->mark(Unmarked);

          if (canApproach && canWithdraw)
            {
              //cout << "\tmust pick direction\n";
              int dx=col-moveState.pendingFrom.col;
              int dy=row-moveState.pendingFrom.row;

              moveState.pendingTo.col=col;
              moveState.pendingTo.row=row;

              markChoice(col+dx,
                         row+dy,
                         Approach);
              markChoice(moveState.pendingFrom.col-dx,
                         moveState.pendingFrom.row-dy,
                         Withdrawal);
              return;
            }

          /*cout << "\tpicked "
               << (canApproach ? "approach" : "withdrawal")
               << "\n";
          */

          fanoronaPlayerMoveMade(player,
                                 moveState.pendingFrom.col,
                                 moveState.pendingFrom.row,
                                 col,row,
                                 canApproach,
                                 &turnIsOver
                                 );
          moveState.reset();
          if (!turnIsOver)
            {
              moveState.pendingFrom.col=col;
              moveState.pendingFrom.row=row;
              cells[col][row]->mark(Possible);
            }
        }
      else
        {
          //cout << "\tinitial click\n";
          if (  fanoronaBoardGetCell(&(getBoard()),
                                     col,row)
              !=fanoronaBoardCellStatePlayer1
                )
            {
              //cout << "\tbut it's not yours\n";
              return;
            }
          moveState.pendingFrom.col=col;
          moveState.pendingFrom.row=row;
          cells[col][row]->mark(Possible);
          return;
        }
    }

  if (turnIsOver<0)
    {
      cerr << "turnIsOver<0\n";
      return;
    }

  if (turnIsOver)
    {
      //cout << "\nturn over\n";
      whoseTurn(0);
      disabled=1;
      //cout << "add_timeout(" << delay << ")\n";
      Fl::add_timeout(delay,::moveComputer,(void*)this);
    }
}

int FanoronaFlBoard::Cell::handle(int event)
{
  switch (event)
    {
    case FL_PUSH:
      parent.cellClicked(*this,col,row);
      return 1;

    default:
      return 0;
    }
}

void FanoronaFlBoard::cellChanged(int col,
                                  int row, 
                                  FanoronaBoardCellState state)
{
  /*
    cout << "cellChanged(" << col << "," << row << ",";
  switch (state)
    {
    case fanoronaBoardCellStateEmpty:
      cout << "Empty";
      break;

    case fanoronaBoardCellStatePlayer1:
      cout << "Player1";
      break;

    case fanoronaBoardCellStatePlayer2:
      cout << "Player2";
      break;
    }

  cout << ")\n";
  */

  Cell* cell=cells[col][row];
  if (cell)
    cell->redraw();
}

void boardCellChanged(FanoronaBoard* board,
                      int col,
                      int row,
                      FanoronaBoardCellState state,
                      void* context)
{
  FanoronaFlBoard* flBoard=(FanoronaFlBoard*)context;
  flBoard->cellChanged(col,row,state);
}

void FanoronaFlBoard::newGame()
{
  eraseChoices();

  if (moveState.pendingFrom.col>=0)
    cells[moveState.pendingFrom.col][moveState.pendingFrom.row]
      ->mark(Unmarked);

  moveState.reset();
  fanoronaPlayerNewGame(player);
  whoseTurn(1);
  disabled=0;
  redraw();
}

void FanoronaFlBoard::gameOver(int humanWon)
{
  disabled=1;

  Fl_Window window(100,50);
  Fl_Return_Button ok(0,0,100,50,
                      (humanWon
                       ? "You win!"
                       : "I win!"
                       )
                      );

  window.label("Game over");
  window.hotspot(&ok); // you must position modal windows
  window.end();
  window.set_modal();
  window.show();
  for (;;) {
    Fl::wait();
    Fl_Widget *o;
    while ((o = Fl::readqueue())) {
      if (o == &ok) {
        return;
      }
    }
  }
}

void FanoronaFlBoard::whoseTurn(int human)
{
  cout << (human ? "Your" : "My") << " turn\n";
}

void FanoronaFlBoard::start()
{
  whoseTurn(1);
}
