/* Copyright (c) 2000 John Stracke <fanorona@thibault.org>
 *
 * This file is part of Fanorona.
 *
 * Fanorona is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any later
 * version. 
 *
 * Fanorona is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
 * the GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with program; see the file COPYING. If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

#include <fanorona/fltk.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/fl_draw.H>

#include <stdlib.h>
#include <iostream.h>

class Label: public Fl_Box {
protected:
  void draw()
  {
    fl_clip(x(),y(),w(),h());
    {
      int x1=x();
      int x2=x1+w();
      int y1=y();
      int y2=y1+h();

      fl_color(color());
      fl_polygon(x1,y1,x1,y2,x2,y2,x2,y1);
    }
    fl_pop_clip();

    Fl_Box::draw();
  }
public:
  Label(int x, int y,
        int w, int h,
        const char* label=0)
    : Fl_Box(x,y,w,h,label)
  {}
};

class Board: public FanoronaFlBoard {
  Fl_Widget* display;
public:
  Board(int x, int y,
        int w, int h,
        const char* label=0)
    : FanoronaFlBoard(x,y,w,h,label),
      display(0)
  {}

  void setDisplay(Fl_Widget* _display)
  {display=_display;}

  virtual void whoseTurn(int human);
};

void Board::whoseTurn(int human)
{
  const char* msg=(human ? "Your turn" : "My turn");

  if (display)
    {
      display->label(msg);
      display->redraw();
    }
}

Board* b;

void quit_cb(Fl_Widget*, void*)
{
  exit(0);
}

void new_game_cb(Fl_Widget*, void*)
{
  b->newGame();
}

Fl_Menu_Item menutable[] = {
  {"Done",0, quit_cb, 0},
  {"New",0, new_game_cb, 0},
  {0}
};

int main(int argc, char* argv[])
{
#ifdef AGENDA
  const int width=160;
  const int height=240;
#else
  const int width=160;
  const int height=150;
#endif

  const int menuHeight=30;
  const int menuY=height-menuHeight;
  const int boardHeight=100;
  const int displayHeight=20;
  const int boardY=(menuY-(boardHeight+displayHeight))/2;

  Fl_Window window(width,height);

  Board board(0,boardY,width,boardHeight);

  Label display(0,boardY+boardHeight,
                width,displayHeight,"Status");
  
  board.setDisplay(&display);

  Fl_Menu_Bar menubar(0,menuY,width,menuHeight);

  b=&board;

  menubar.menu(menutable);

  window.end();

  window.label("Fanorona");

  window.show(argc, argv);
  
  board.start();

  return Fl::run();
}
